defmodule Chronoscope.NTS.Behaviour do
  @callback(key_establishment(host :: String.t(), port :: integer()) :: {:ok, Map.t()}, {:error, any()})
end

defmodule Chronoscope.NTS do
  @behaviour Chronoscope.NTS.Behaviour

  require Logger

  alias Chronoscope.NTS

  def healthy?() do
    true
  end

  def list() do
    Chronoscope.DynamicSupervisor
    |> DynamicSupervisor.which_children()
    |> Enum.map(fn {_, pid, _, _} -> GenServer.call(pid, :list) end)
  end

  def key_establishment(host, port) do
    # todo don't use atoms for this
    name = String.to_atom("#{host}:#{port}")

    if GenServer.whereis(name) == nil do
      DynamicSupervisor.start_child(Chronoscope.DynamicSupervisor, {NTS.Client, host: host, port: port, name: name})
    end

    GenServer.call(name, :key_establishment)
  end
end
